@@ -644,12 +644,15 @@ static void soundcard_setup_write_switch(FILE * out, const char *space, int inte
 		break;
 	case SND_SW_TYPE_BYTE:
 		s = "byte";
-		sprintf(v, "%u", (unsigned int) sw->value.data8[0]);
+		if (sw->subtype == SND_SW_SUBTYPE_HEXA) {
+			sprintf(v, "0x%x", (unsigned int) sw->value.data8[0]);
+		} else {
+			sprintf(v, "%u", (unsigned int) sw->value.data8[0]);
+		}
 		break;
 	case SND_SW_TYPE_WORD:
 		s = "word";
-		if (interface == SND_INTERFACE_CONTROL &&
-		    !strcmp(sw->name, SND_CTL_SW_JOYSTICK_ADDRESS)) {
+		if (sw->subtype == SND_SW_SUBTYPE_HEXA) {
 		    	sprintf(v, "0x%x", (unsigned int) sw->value.data16[0]);
 		} else {
 			sprintf(v, "%u", (unsigned int) sw->value.data16[0]);
@@ -657,7 +660,11 @@ static void soundcard_setup_write_switch(FILE * out, const char *space, int inte
 		break;
 	case SND_SW_TYPE_DWORD:
 		s = "dword";
-		sprintf(v, "%u", sw->value.data32[0]);
+		if (sw->subtype == SND_SW_SUBTYPE_HEXA) {
+			sprintf(v, "0x%x", sw->value.data32[0]);
+		} else {
+			sprintf(v, "%u", sw->value.data32[0]);
+		}
 		break;
 	case SND_SW_TYPE_USER:
 		s = "user";
@@ -666,6 +673,8 @@ static void soundcard_setup_write_switch(FILE * out, const char *space, int inte
 		s = "list";
 		sprintf(v, "%u", sw->value.item_number);
 		break;
+	case SND_SW_TYPE_USER_READ_ONLY:
+		return;
 	default:
 		s = "unknown";
 	}
@@ -726,11 +735,11 @@ static void soundcard_setup_write_switch(FILE * out, const char *space, int inte
 		fprintf(out, v);
 		if (sw->type < 0 || sw->type > SND_SW_TYPE_LIST_ITEM) {
 			/* TODO: some well known types should be verbose */
-			fprintf(out, "rawdata(");
+			fprintf(out, "rawdata(@");
 			for (idx = 0; idx < 31; idx++) {
-				fprintf(out, "@%02x:", sw->value.data8[idx]);
+				fprintf(out, "%02x:", sw->value.data8[idx]);
 			}
-			fprintf(out, "%02x@)\n", sw->value.data8[31]);
+			fprintf(out, "%02x@)", sw->value.data8[31]);
 		}
 	}
       __end:
